/***********************************************************************************
    TwoCitronas Klickowanna
    Copyright (C) 2012 Two Citronas, http://two-citronas.com/, bwpow@two-citronas.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see http://www.gnu.org/licenses/.
***********************************************************************************/

#include <MAUtil/Moblet.h>
#include <MAUtil/Environment.h>
#include <ma.h>
#include <maapi.h>
#include <mavsprintf.h>
#include <mastdlib.h>

#include "MAHeaders.h"

using namespace MAUtil;


typedef struct {
	int x,y,w,h,dx,dy;
}VEC;

typedef struct {
	short x,y;
} XY;

typedef struct {
	float x,y;
} XYf;

#define MAX_VEC 6

const VEC vec[MAX_VEC]={
		{0  ,  0,450,450,  0,  0},
		{205,205, 40, 40,  0,  0},
		{270, 60, 60, 50,-10, 12},
		{300,330,100, 20,-12,-20},
		{ 70,320, 30, 60, 15,-13},
		{ 70, 70, 60, 60, 17, 11}
};

#define ID_ARENA 0
#define ID_PLAYER 1
#define ID_VEC 2

#define ALIGN_LEFT 0
#define ALIGN_RIGHT 1
#define ALIGN_CENTER 2

#define ZOOM(x) ((int)(x*zoom))

class MyClickowanna : public Moblet, public TimerListener
{
public:
	MyClickowanna()
	{
		p_size=maGetTextSize("M");
		p_text_lines=0;
		splash=true;

		resetGame();
		prepareSplashScreen();
		drawScreen();
		addTimer(this,40,0);
	}

	void customEvent(const MAEvent& event) {
		if(event.type==EVENT_TYPE_SCREEN_CHANGED){
			resetGame();
			drawScreen();
		}
	}

	void focusLost()
	{
		close();
	}

	void focusGained()
	{
		drawScreen();
	}

	void closeEvent()
	{
		focusLost();
	}

	int drawText(const int id,const int num,const int align=ALIGN_CENTER,const int yy=-1)
	{
		MAExtent size;
		int y=yy,x;
		if(y<0){
			y=by+(bh-(EXTENT_Y(p_size)+5)*(num+1))/2+5;
		}
		for(int i=0;i<num;i++){
			size=maGetTextSize(p_text[i+id]);
			x=bx+5;
			if(align==ALIGN_RIGHT){
				x=bx+bw-5-EXTENT_X(size);
			}
			if(align==ALIGN_CENTER){
				x=bx+(bw-EXTENT_X(size))/2;
			}

			maDrawText(x,y,p_text[i+id]);
			y+=EXTENT_Y(p_size)+5;
		}
		return y;
	}

	void drawSplashScreen(void)
	{
		bx=(w-bw)/2;
		by=(h-bh)/2;

		maSetClipRect(bx,by,bw,bh);

		maSetColor(0);
		maFillRect(bx,by,bw,bh);
		maSetColor(0xFFFFFF);
		maFillRect(bx+3,by+3,bw-6,bh-6);

		maSetColor(0);

		drawText(0,p_text_lines);
	}

	void prepareSplashScreen(void)
	{
		p_text_lines=0;
		strcpy(p_text[p_text_lines++],"Klickowanna");
		strcpy(p_text[p_text_lines++],"by Samuel 'bwpow' Kupka, http://bwpow.eu/");
		p_text[p_text_lines++][0]='\0';
		strcpy(p_text[p_text_lines++],"Copyright (C) 2012 Two Citronas");
		strcpy(p_text[p_text_lines++],"http://two-citronas.com/");
		p_text[p_text_lines++][0]='\0';
		strcpy(p_text[p_text_lines++],"This program is free software: you can redistribute it and/or modify");
		strcpy(p_text[p_text_lines++],"it under the terms of the GNU General Public License as published by");
		strcpy(p_text[p_text_lines++],"the Free Software Foundation, either version 3 of the License, or");
		strcpy(p_text[p_text_lines++],"(at your option) any later version.");

		bw=EXTENT_X(p_size)*(strlen(p_text[6])+10);
		bh=(EXTENT_Y(p_size)+5)*(p_text_lines+5);
	}

	void drawFailedBox(void)
	{
		bx=(w-bw)/2;
		by=(h-bh)/2;
		maSetClipRect(bx,by,bw,bh);

		maSetColor(0);
		maFillRect(bx,by,bw,bh);
		maSetColor(0xFFFFFF);
		maFillRect(bx+3,by+3,bw-6,bh-6);

		maSetColor(0);

		drawText(0,3);

		maSetClipRect(px,py,sizes[ID_ARENA].x,sizes[ID_ARENA].y);
	}

	void drawScreen(void)
	{
		if(splash){
			drawSplashScreen();
			maUpdateScreen();
			return;
		}

		maSetColor(failed?0xBBBBCC:0xFFFFFF);
		maFillRect(px,py,sizes[ID_ARENA].x,sizes[ID_ARENA].y);

		maSetColor(failed?0x990000:0xFF0000);
		maFillRect(px+ZOOM(pos[ID_PLAYER].x),py+ZOOM(pos[ID_PLAYER].y),sizes[ID_PLAYER].x,sizes[ID_PLAYER].y);

		maSetColor(failed?0x000099:0x0000FF);
		for(int i=ID_VEC;i<MAX_VEC;i++){
			maFillRect(px+ZOOM(pos[i].x),py+ZOOM(pos[i].y),sizes[i].x,sizes[i].y);
		}

		if(failed){
			drawFailedBox();
		}

		maUpdateScreen();
	}

	void startGame(void)
	{
		resetGame();
		failed=false;
		game=true;
		timeStart=maGetMilliSecondCount();
	}

	void failGame(void)
	{
		timeEnd=maGetMilliSecondCount();
		game=false;
		failed=true;

		sprintf(p_text[0],"Only %g seconds?",(float)(timeEnd-timeStart)/1000);
		p_text[1][0]='\0';
		strcpy(p_text[2],"Press here to reset...");

		bw=EXTENT_X(p_size)*(strlen(p_text[0])+10);
		bh=(EXTENT_Y(p_size)+5)*(3+5);
		bx=(w-bw)/2;
		by=(h-bh)/2;
	}

	void resetGame(void)
	{
		MAExtent size;

		game=false;
		failed=false;
		speed=0.17f;

		size=maGetScrSize();
		w=EXTENT_X(size);
		h=EXTENT_Y(size);

		zoom= (float)(w-20)/(float)(vec[ID_ARENA].w) <? (float)(h-20)/(float)(vec[ID_ARENA].h);
		for(int i=0;i<MAX_VEC;i++){
			sizes[i].x=(int)((float)vec[i].w*zoom);
			sizes[i].y=(int)((float)vec[i].h*zoom);
			pos[i].x=(float)vec[i].x;
			pos[i].y=(float)vec[i].y;
			delta[i].x=(float)vec[i].dx;
			delta[i].y=(float)vec[i].dy;
		}
		px=(w-sizes[ID_ARENA].x)/2;
		py=(h-sizes[ID_ARENA].y)/2;

		maSetClipRect(0,0,w,h);

		size=maGetImageSize(BACK);
		for(int x=0;x<w;x+=EXTENT_X(size)){
			for(int y=0;y<h;y+=EXTENT_Y(size)){
				maDrawImage(BACK,x,y);
			}
		}

		if(!splash){
			maSetColor(0);
			maFillRect(px-10,py-10,10,sizes[ID_ARENA].y+20);
			maFillRect(sizes[ID_ARENA].x+px,py-10,10,sizes[ID_ARENA].y+20);

			maFillRect(px,py-10,sizes[ID_ARENA].x,10);
			maFillRect(px,sizes[ID_ARENA].y+py,sizes[ID_ARENA].x,10);

			maSetClipRect(px,py,sizes[ID_ARENA].x,sizes[ID_ARENA].y);
		}
	}

	void keyPressEvent(int keyCode, int nativeCode)
	{
		if (MAK_BACK == keyCode || MAK_0 == keyCode){
			close();
		}
	}

	void checkForCollisions(void)
	{
        if(pos[ID_PLAYER].x<=0||pos[ID_PLAYER].y<=0||pos[ID_PLAYER].x+vec[ID_PLAYER].w>=vec[ID_ARENA].w||pos[ID_PLAYER].y+vec[ID_PLAYER].h>=vec[ID_ARENA].h){
			failGame();
			return;
		}

        int l1,l2,r1,r2,t1,t2,b1,b2;
		l1=(int)pos[ID_PLAYER].x;
		r1=(int)pos[ID_PLAYER].x+vec[ID_PLAYER].w;
		t1=(int)pos[ID_PLAYER].y;
		b1=(int)pos[ID_PLAYER].y+vec[ID_PLAYER].h;

		for(int i=2;i<6;i++){
			l2=(int)pos[i].x;
			r2=(int)pos[i].x+vec[i].w;
			t2=(int)pos[i].y;
			b2=(int)pos[i].y+vec[i].h;

	        if(b1<t2||t1>b2||r1<l2||l1>r2) continue;
			failGame();
			return;
		}

	}

	void moveVeci(void)
	{
		for(int i=ID_VEC;i<=MAX_VEC;i++){
			pos[i].x+=delta[i].x*speed;
			pos[i].y+=delta[i].y*speed;
			if(pos[i].x<=0) delta[i].x=-delta[i].x;
			if(pos[i].y<=0) delta[i].y=-delta[i].y;
			if(pos[i].x+vec[i].w>=vec[ID_ARENA].w) delta[i].x=-delta[i].x;
			if(pos[i].y+vec[i].h>=vec[ID_ARENA].h) delta[i].y=-delta[i].y;
		}
		if(speed<1.2f){
			speed+=0.0018f;
		}
		checkForCollisions();
		drawScreen();
	}

	void runTimerEvent(void)
	{
		if(game&&!failed){
			moveVeci();
		}
	}

	void pointerPressEvent(MAPoint2d point)
	{
		if(!game&&!failed){
			int x=(int)((float)(point.x-px)/zoom)-vec[ID_PLAYER].w/2;
			int y=(int)((float)(point.y-py)/zoom)-vec[ID_PLAYER].h/2;
			if(abs(vec[ID_PLAYER].x-x)<vec[ID_PLAYER].w&&abs(vec[ID_PLAYER].y-y)<vec[ID_PLAYER].h){
				startGame();
				drawScreen();
			}
		}
		if(!game&&failed){
			if(point.x>bx&&point.y>by&&point.x<bx+bw&&point.y<by+bh){
				failed=false;
				resetGame();
				drawScreen();
			}
		}
		if(!game&&!failed&&splash){
			if(point.x>bx&&point.y>by&&point.x<bx+bw&&point.y<by+bh){
				failed=false;
				splash=false;
				resetGame();
				drawScreen();
			}
		}
	}

	void pointerMoveEvent(MAPoint2d point)
	{
		if(game&&!failed){
			pos[1].x=(float)(point.x-px)/zoom-vec[1].w/2.0f;
			pos[1].y=(float)(point.y-py)/zoom-vec[1].h/2.0f;
			checkForCollisions();
			drawScreen();
		}
	}

	void pointerReleaseEvent(MAPoint2d point)
	{
		if(game&&!failed){
			failGame();
			drawScreen();
		}
	}

private:
	int w,h;
	int bx,by,bw,bh;
	int px,py;
	int timeStart,timeEnd;
	bool game,failed,splash;
	float zoom,speed;
	XY sizes[MAX_VEC];
	XYf pos[MAX_VEC],delta[MAX_VEC];

	char p_text[16][256];
	int p_text_lines;
	MAExtent p_size;
};

extern "C" int MAMain()
{
	maScreenSetOrientation(SCREEN_ORIENTATION_LANDSCAPE);
	maScreenSetFullscreen(1);
	maScreenStateEventsOn();

	Moblet::run(new MyClickowanna());
	return 0;
}
